package com.example.mathseduc.ui

import android.content.Context
import android.content.Intent
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.ArrowBack
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.DisposableEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.example.mathseduc.CreateLobbyActivity
import com.example.mathseduc.MainActivity
import com.example.mathseduc.ServerDetailsActivity
import com.example.mathseduc.controllers.ControllerChapter
import com.example.mathseduc.controllers.ControllerLobby
import com.example.mathseduc.controllers.ControllerUtiliser
import com.example.mathseduc.models.Lobby
import com.example.mathseduc.ui.theme.Colors
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import okhttp3.MultipartBody

@Composable
fun MultiPage() {
    val context = LocalContext.current
    var selectedItem by rememberSaveable { mutableStateOf<Lobby?>(null) }
    var lobbyList by remember { mutableStateOf<List<Lobby>>(emptyList()) }
    val scope = rememberCoroutineScope()
    var refreshState by remember { mutableStateOf(true) }

    // Fonction pour actualiser la liste des lobbies
    suspend fun refreshLobbyList() {
        val refreshedLobbyList = withContext(Dispatchers.IO) {
            ControllerLobby.getLobbies() ?: emptyList()
        }
        lobbyList = refreshedLobbyList
        Log.e("MainActivity", "Refreshed Lobby List")
    }

    DisposableEffect(refreshState) {
        val refreshRunnable = {
            scope.launch(Dispatchers.IO) {
                try {
                    while (refreshState) {
                        delay(5000)
                        refreshLobbyList()
                    }
                } catch (e: Exception) {
                    Log.e("MainActivity", "Error refreshing lobbies: ${e.message}")
                }
            }
        }
        refreshRunnable.invoke()

        onDispose {
            Toast.makeText(context, "Au REVOIR UwU", Toast.LENGTH_SHORT).show()
            refreshState = false
        }
    }

    Box(
        modifier = Modifier
                .fillMaxSize()
                .padding(16.dp)
    ) {
        IconButton(
            onClick = {
                val intent = Intent(context, MainActivity::class.java)
                context.startActivity(intent)
            },
            modifier = Modifier
                .align(Alignment.TopStart)
        ) {
            Icon(
                Icons.Rounded.ArrowBack,
                "arrow_back",
                tint = Colors.White
            )
        }

        Column(
            modifier = Modifier
                    .fillMaxSize()
                    .padding(top = 48.dp)
        ) {
            Row(
                modifier = Modifier
                        .fillMaxWidth()
                        .padding(vertical = 10.dp),
                verticalAlignment = Alignment.CenterVertically
            ) {
                Text(
                    text = "Liste des lobbies",
                    color = Colors.White,
                    modifier = Modifier
                            .weight(1f)
                            .padding(start = 20.dp)
                )
                Button(
                    onClick = {
                        refreshState = false
                        val intent = Intent(context, CreateLobbyActivity::class.java)
                        context.startActivity(intent)
                    },
                    shape = RoundedCornerShape(15),
                    modifier = Modifier
                            .weight(1f)
                            .padding(end = 20.dp),
                    colors = ButtonDefaults.buttonColors(Colors.Blue)
                ) {
                    Text(text = "Ajouter un lobby")
                }
            }

            LazyColumn(
                modifier = Modifier
                        .fillMaxSize()
                        .weight(0.85f)
            ) {
                items(lobbyList) { lobby ->
                    LobbyItem(lobby, selectedItem == lobby) {
                        selectedItem = it
                        refreshState = false

                        if (ControllerLobby.getNbPlayerInLobby(selectedItem!!.id) < selectedItem!!.nbplayers) {
                            // Créer un Utiliser si le lobby n'est pas plein

                            scope.launch {
                                createUtiliserForLobby(context, selectedItem!!)
                            }
                        } else {
                            // Afficher un message Toast si le lobby est plein
                            Toast.makeText(
                                context,
                                "Oh nan, le serveur est déjà plein ! Réessayer plus tard.",
                                Toast.LENGTH_SHORT
                            ).show()
                        }
                    }
                }
            }
        }
    }
}

// Fonction pour créer un Utiliser pour le lobby sélectionné
private suspend fun createUtiliserForLobby(context: Context, lobby: Lobby) {
    withContext(Dispatchers.IO) {
        val formDataBuilder = MultipartBody.Builder().setType(MultipartBody.FORM)
        formDataBuilder.addFormDataPart("idplayer", MainActivity.idPlayerConnected.toString())
        formDataBuilder.addFormDataPart("idlobby", lobby.id.toString())
        formDataBuilder.addFormDataPart("playertime", "0")

        ControllerUtiliser.createUtiliserByIdLobby(formDataBuilder)

        // Naviguer vers l'activité ServerDetails avec les détails du lobby
        val intent = Intent(context, ServerDetailsActivity::class.java)
        intent.putExtra("serverName", lobby.name)
        intent.putExtra("lobbyId", lobby.id)
        context.startActivity(intent)
    }
}

@Composable
fun LobbyItem(lobby: Lobby, isSelected: Boolean, onItemClick: (Lobby) -> Unit) {
    val context = LocalContext.current
    var showDialog by rememberSaveable { mutableStateOf(false) }
    Row(
        modifier = Modifier
                .fillMaxWidth()
                .padding(8.dp)
                .clickable {
                    if (lobby.password.isNotEmpty()) {
                        showDialog = true
                    } else {
                        onItemClick(lobby)
                    }
                }
                .background(if (isSelected) Colors.Orange else Colors.Grey)
                .padding(16.dp)
                .clip(RoundedCornerShape(8.dp))
    ) {
        Text(
            text = lobby.name,
            fontSize = 18.sp,
            color = Colors.White,
            modifier = Modifier.weight(1f)
        )
        Text(
            text = "${ControllerLobby.getNbPlayerInLobby(lobby.id)}/${lobby.nbplayers}",
            fontSize = 18.sp,
            color = Colors.White,
            modifier = Modifier.weight(1f)
        )
        Text(
            text = ControllerChapter.getChapterNameById(lobby.idchapter).toString(),
            fontSize = 18.sp,
            color = Colors.White,
            modifier = Modifier.weight(1f)
        )
        Text(
            text = lobby.difficulty.toString(),
            fontSize = 18.sp,
            color = Colors.White,
            modifier = Modifier.weight(1f)
        )

        if (showDialog) {
            ConfirmDialog(
                    onDismiss = { showDialog = false },
                onConfirm = { password ->
                    println("Password input : $password")
                    if (password == lobby.password) {
                    showDialog = false
                    onItemClick(lobby)
                } else {
                    Toast.makeText(context, "Wrong password!", Toast.LENGTH_SHORT).show()
                }
            }
            )
        }
    }
}

@Composable
fun ConfirmDialog(onDismiss: () -> Unit, onConfirm: (String) -> Unit) {
    val context = LocalContext.current

    var password by remember { mutableStateOf("") }

    AlertDialog(
            onDismissRequest = {
                onDismiss()
            },
            title = { Text("Enter Password") },
            confirmButton = {
                Button(
                        onClick = {
                            onConfirm(password)
                        }
                ) {
                    Text("Confirm")
                }
            },
            dismissButton = {
                Button(
                        onClick = {
                            onDismiss()
                        }
                ) {
                    Text("Dismiss")
                }
            },
            text = {
                Column(
                        modifier = Modifier
                                .fillMaxWidth()
                                .padding(8.dp)
                ) {

                    OutlinedTextField(
                            value = password,
                            onValueChange = { password = it },
                            label = { Text("Password") },
                            visualTransformation = PasswordVisualTransformation(),
                            keyboardOptions = KeyboardOptions.Default.copy(
                                    keyboardType = KeyboardType.Password
                            ),
                            modifier = Modifier
                                    .fillMaxWidth()
                                    .padding(8.dp)
                    )
                }
            }
    )
}